#include "aaroniahttp.h"

AaroniaHTTP::AaroniaHTTP(quint32 httpPort, QString address)
    : mAddress(address), mPort(httpPort)
{
    mNetworkAccessManager = new QNetworkAccessManager(this);
}

QNetworkReply * AaroniaHTTP::StartStreaming(bool on)
{
    QJsonDocument	jdoc(    {
                                 {"start", on},
                                 {"type" , "streaming"}
                             });

    return HTTPPutRequest("/control", jdoc );
}

QNetworkReply *AaroniaHTTP::StartRecording(bool on)
{
    QJsonDocument	jdoc(    {
                                 {"start", on},
                                 {"type" , "recording"}
                             });

    return HTTPPutRequest("/control", jdoc );
}

QNetworkReply *AaroniaHTTP::getConfig()
{
    return HTTPGetRequest("/remoteconfig");
}

QNetworkReply * AaroniaHTTP::setConfig(QJsonDocument config)
{
    return HTTPPutRequest("/remoteconfig", config );
}

QNetworkReply * AaroniaHTTP::getHealthStatus()
{
    return HTTPGetRequest("/healthstatus");
}

QNetworkReply *AaroniaHTTP::getDataStream(quint32 port, QString parameter)
{
    QUrl	url(mAddress + QString::number(port) + "/stream?"+ parameter);
    QNetworkRequest	request(url);
    return mNetworkAccessManager->get(request);
}

QNetworkReply *AaroniaHTTP::HTTPGetRequest(QString endPoint)
{
   QUrl	url(mAddress + QString::number(mPort) + endPoint);
   QNetworkRequest	request(url);
   return mNetworkAccessManager->get(request);
}

QNetworkReply *AaroniaHTTP::HTTPPutRequest(QString endPoint, QJsonDocument data)
{
    QNetworkRequest request(QUrl(mAddress + QString::number(mPort) + endPoint));
    request.setHeader(QNetworkRequest::ContentTypeHeader, QString("application/json"));

    return mNetworkAccessManager->put(request, data.toJson());
}

QNetworkReply *AaroniaHTTP::HTTPPostRequest(QString endPoint, QByteArray &data)
{
    QNetworkRequest request(QUrl(mAddress + QString::number(mPort) + endPoint));
    request.setHeader(QNetworkRequest::ContentTypeHeader, QString("application/json"));

    return mNetworkAccessManager->post(request, data);
}

QString AaroniaHTTP::getAddress() const
{
    return mAddress;
}

void AaroniaHTTP::setAddress(const QString &address)
{
    mAddress = address;
}

quint32 AaroniaHTTP::getPort() const
{
    return mPort;
}

void AaroniaHTTP::setPort(const quint32 &port)
{
    mPort = port;
}

