#include "mainwindow.h"
#include "ui_mainwindow.h"

#include <QJsonArray>
#include <QDebug>

MainWindow::MainWindow(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::MainWindow)
{
    ui->setupUi(this);

    mRequest = nullptr;
    mRequestStream1 = nullptr;
    mRequestStream2 = nullptr;
    mRemoteMission = new AaroniaHTTP( ui->spinBoxPort->value() );
}

MainWindow::~MainWindow()
{
    delete ui;
}


void MainWindow::on_pushButtonStart_clicked(bool checked)
{
    if(mRequest != nullptr)
        return;

    mRequest = mRemoteMission->StartStreaming(checked);

    connect(mRequest, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onError(QNetworkReply::NetworkError)));
    connect(mRequest, SIGNAL(finished()), this, SLOT(onFinished()));
}


void MainWindow::on_pushButtonStartRecording_clicked(bool checked)
{
    if(mRequest != nullptr)
        return;

    mRequest = mRemoteMission->StartRecording(checked);

    connect(mRequest, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onError(QNetworkReply::NetworkError)));
    connect(mRequest, SIGNAL(finished()), this, SLOT(onFinished()));
}


void MainWindow::on_pushButtonGetConfig_clicked()
{
    if(mRequest != nullptr)
        return;

    mRequest = mRemoteMission->getConfig();

    connect(mRequest, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onError(QNetworkReply::NetworkError)));
    connect(mRequest, SIGNAL(finished()), this, SLOT(onFinished()));
}

void MainWindow::on_pushButtonGetHealthStatus_clicked()
{
    if(mRequest != nullptr)
        return;

    mRequest = mRemoteMission->getHealthStatus();

    connect(mRequest, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onError(QNetworkReply::NetworkError)));
    connect(mRequest, SIGNAL(finished()), this, SLOT(onFinished()));
}

void MainWindow::on_pushButtonSetConfig_clicked()
{
    if(mRequest != nullptr)
        return;

    QJsonParseError	error;
    QJsonDocument	jdoc = QJsonDocument::fromJson( ui->plainTextEditSetConfig->toPlainText().toLocal8Bit() , &error);

    if (error.error == QJsonParseError::NoError)
    {
        mRequest = mRemoteMission->setConfig(   jdoc );

        connect(mRequest, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onError(QNetworkReply::NetworkError)));
        connect(mRequest, SIGNAL(finished()), this, SLOT(onFinished()));
    }
    else
    {
        qDebug() << "Json Parse Error: " + error.errorString();
        ui->plainTextEditSetConfig->setPlainText(error.errorString());
    }
}


void MainWindow::on_pushButtonGetStream1_clicked(bool checked)
{
    if(!checked
            & (mRequestStream1 != nullptr) )
    {
        mRequestStream1->abort();
        return;
    }

    if(mRequestStream1 != nullptr)
        return;

    mRequestStream1 = mRemoteMission->getDataStream(ui->spinBoxPortStream1->value(),ui->lineEditStream1Parameter->text());

    connect(mRequestStream1, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onStream1Error(QNetworkReply::NetworkError)));
    connect(mRequestStream1, SIGNAL(finished()), this, SLOT(onStream1Finished()));
    connect(mRequestStream1, SIGNAL(readyRead()), this, SLOT(onStream1ReadyRead()));
}

void MainWindow::on_pushButtonGetStream2_clicked(bool checked)
{
    if(!checked
            & (mRequestStream2 != nullptr) )
    {
        mRequestStream2->abort();
        return;
    }

    if(mRequestStream2 != nullptr)
        return;

    mRequestStream2 = mRemoteMission->getDataStream(ui->spinBoxPortStream2->value(), ui->lineEditStream2Parameter->text());

    connect(mRequestStream2, SIGNAL(error(QNetworkReply::NetworkError)), this, SLOT(onStream2Error(QNetworkReply::NetworkError)));
    connect(mRequestStream2, SIGNAL(finished()), this, SLOT(onStream2Finished()));
    connect(mRequestStream2, SIGNAL(readyRead()), this, SLOT(onStream2ReadyRead()));
}

void MainWindow::on_spinBoxPort_valueChanged(int arg1)
{
    mRemoteMission->setPort(arg1);
}

void MainWindow::on_lineEditAddress_editingFinished()
{
    mRemoteMission->setAddress(ui->lineEditAddress->text());
}


void MainWindow::onError(QNetworkReply::NetworkError code)
{
    qDebug() << "Network Error: " + mRequest->errorString() ;
    ui->plainTextEditResponse->setPlainText(  mRequest->errorString() );
}

void MainWindow::onFinished()
{
    QTextStream	qerr(stderr);
    qerr << "Finished: " + mRequest->errorString();

    QByteArray	data = mRequest->readAll();

    QJsonParseError	error;
    QJsonDocument	jdoc = QJsonDocument::fromJson(data, &error);

    if (error.error == QJsonParseError::NoError)
    {
        ui->plainTextEditResponse->setPlainText(  jdoc.toJson() );

        QJsonObject config = jdoc["config"].toObject();
        QJsonArray blocks = jdoc["config"]["items"].toArray();

        for (int i = 0; i < blocks.count() ; i++ ) {

            if(blocks.at(i)["name"].toString() == "Block_Spectran_V6B_0")
            {
                QJsonArray blockItems = blocks.at(i)["items"].toArray();

                //  blocks.it
            }

        }
    }
    else
        qDebug() << "Json Parse Error: " + error.errorString();

    mRequest->deleteLater();
    mRequest = nullptr;
}


void MainWindow::onReadyRead()
{
    QByteArray	data = mRequest->readAll();

    ui->plainTextEditResponse->setPlainText(  data );
}

void MainWindow::onStream1Error(QNetworkReply::NetworkError code)
{
    qDebug() << "Network Error: " << mRequestStream1->errorString() ;
    ui->plainTextEditStream1->setPlainText(  mRequestStream1->errorString() );
}

void MainWindow::onStream1Finished()
{
    qDebug() << "Finished: " + mRequestStream1->errorString();

    mRequestStream1->deleteLater();
    mRequestStream1 = nullptr;
}

void MainWindow::onStream1ReadyRead()
{
    ui->plainTextEditStream1->setPlainText(  mRequestStream1->readAll() );
}

void MainWindow::onStream2Error(QNetworkReply::NetworkError code)
{
    qDebug() << "Network Error: " << mRequestStream2->errorString() ;
    ui->plainTextEditStream2->setPlainText(  mRequestStream2->errorString() );
}

void MainWindow::onStream2Finished()
{
    qDebug() << "Finished: " + mRequestStream2->errorString();

    mRequestStream2->deleteLater();
    mRequestStream2 = nullptr;
}

void MainWindow::onStream2ReadyRead()
{
    ui->plainTextEditStream2->setPlainText(  mRequestStream2->readAll() );
}
